# -----------------------------------------------------------------------------
# Programm         : B a s i s . p m
# -----------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 10. Februar 2004
# Zuletzt gendert : 12. Februar 2004
# Aufgabe          : Basis Klasse, von der alle meine Klassen erben.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Globale private Datei-Variablen:
# -----------------------------------------------------------------------------
my $Debug  = 0;


# -----------------------------------------------------------------------------
# Klasse           : B a s i s
# abgeleitet von   : none
# -----------------------------------------------------------------------------
# Autor            : Christian Dhl
# Beschreibung     : Eine Basis-Klasse.
# Attribute        : %
# Klassen-Methoden : new       - Erstellt ein Objekt dieser Klasse.
# Objekt-Methoden  : _property - interne Hilfsmethode fr Getter / Setter
#                    clone     - Erzeugt ein geclontes Objekt (eine Kopie,
#                                die referenzierte Memberhashwerte richtig
#                                kopiert und nicht nur eine Kopie der Referenz
#                                anlegt.
# -----------------------------------------------------------------------------
# 0.0.1 - 10.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------
package Graph::GED::Basis;

use strict;
use warnings;


sub new {
# -----------------------------------------------------------------------------
# Methode      : n e w
# Klasse       : Graph::GED::Basis
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Erzeugt ein neues Basis-Objekt (oder ein Objekt einer Klasse,
#                die von dieser Klasse abgeleitet wurde und uns bittet, das
#                Objekt fr sie zu erzeugen).
# Parameter    : 1) Klasse
#                2) ev. gewnschte Klasse (im Fall einer abgeleiteten Klasse)
#                Und ggf. Attribute: siehe Klassenbeschreibung oben
# Rckgabewert : Das geblesste neue Objekt.
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $ref   = shift;
    my $class = (ref $ref) || $ref;
                       # Die Klasse, von der new aufgerufen wurde:
                       #     my $object = Graph->new(...);
                       # oder
                       #     my $object = new Graph(...);
                       #
                       # Ev. wurde auch
                       #
                       #     $basis->new() aufgerufen, dafr
                       #
                       # die zweite Zeile


    my $wantclass;     # Dies ist die Klasse, in welche das neue Objekt ge-
                       # blessed werden soll.
                       # Falls die Methode new mit einer gerade Anzahl an
                       # Parametern aufgerufen wurde (nachdem $class schon ent-
                       # nommen wurde), wurde keine gewnschte Klasse angege-
                       # ben. In diesem Fall verwenden wir $class.
                       # Anderenfalls verwenden wir die bergebene Klasse.

    my $n = scalar @_; # Bestimmt die Anzahl an verbliebenen Parametern.

    #--------------------------------------------------------------------------
    # Falls noch Parameter da sind und ihre Anzahl ungerade ist, dann mchte
    # der Aufrufer, dass wir das Objekt auf die gewnschte Klasse blessen, die
    # als nchster Parameter angegeben ist.
    # Also holen wir sie mit shift aus den Aufrufparametern und legen sie
    # in $wantclass ab.
    #--------------------------------------------------------------------------
    if ($n > 0 and ($n >> 1) << 1 != $n) {
        $wantclass = shift;
    }
    #--------------------------------------------------------------------------
    # Anderenfalls wurde keine gewnschte Klasse angegeben und wir verwenden
    # $class aus dem ersten Parameter:
    #--------------------------------------------------------------------------
    else {
        $wantclass = $class;
    }

    #--------------------------------------------------------------------------
    # Ausgabe der Aufrufparameter:
    #--------------------------------------------------------------------------
    print "Graphen::GED::Basis::new\n",
          "\tclass      = '$class'\n",
          "\twant class = '$wantclass'\n",
          "\tn          = $n\n",
          "\tParameter  = @_\n",
          if $Debug > 1;

    #--------------------------------------------------------------------------
    # Wir erzeugen ein Objekt in der gewnschten Klasse (eine geblessde Refe-
    # renz auf einen ev. leeren anonymen Hash):
    #--------------------------------------------------------------------------
    my $self = bless({@_}, $wantclass);

    #--------------------------------------------------------------------------
    # Hier knnte man noch Attribute initialisieren:
    #--------------------------------------------------------------------------
    #$self->x(-1);

    #--------------------------------------------------------------------------
    # ... und geben das erzeugte Objekt zurck:
    #--------------------------------------------------------------------------
    return $self;

} # sub Basis::new


sub clone {
# -----------------------------------------------------------------------------
# Methode      : c l o n e
# Klasse       : Graph::GED::Basis
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Erzeugt ein geclontes Objekt (eine Kopie, die referenzierte
#                Memberhashwerte richtig kopiert und nicht nur eine Kopie der
#                Referenz anlegt.
# Parameter    : Ev. ein von einer abgeleiteten Klasse nach unten durchgereich-
#                tes Objekt, das wir wiederum durchreichen sollen und an-
#                schlieend seine in dieser Klasse hinzugefgten Kontainer
#                kopieren sollen.
# Rckgabewert : Das geclonte Objekt.
# -----------------------------------------------------------------------------
# 0.0.1 - 10.02.2004 - CD - erstellt
# 0.0.2 - 11.02.2004 - CD - Funktionalitt fertig gestellt
# -----------------------------------------------------------------------------
    my $self = shift;
    my $ursp = shift;
    my $clone;


    print "Graph::GED::Basis::Clone\n" if $Debug;

    #--------------------------------------------------------------------------
    # Bekommen wir von "oben" ein Objekt?
    #--------------------------------------------------------------------------
    if (defined $ursp) {
        $clone = $ursp;
    }
    #--------------------------------------------------------------------------
    # Falls nicht erzeugen wir das Objekt und kopieren unsere Daten flach:
    #--------------------------------------------------------------------------
    else {
        $clone = $self->new();
        %$clone = %$self;
    }

    #--------------------------------------------------------------------------
    # Da wir Basisklasse sind, gibt es keine geerbeten Teile
    #--------------------------------------------------------------------------
    #$self->SUPER::clone($clone);

    #--------------------------------------------------------------------------
    # Dann Kontainer kopieren:
    # Gibt es im Moment nicht
    #--------------------------------------------------------------------------
    #my @kanten = @{$self->_kanten()}; # kopiert die Inhalte
    ## Hier mssen noch alle Elemente des Arrays gecloned werden, da das ja auch
    ## Referenzen sind!
    #$clone->_kanten(\@kanten);        # Referenz auf Kopie ablegen


    return $clone;
} # sub clone


sub _property {

# -----------------------------------------------------------------------------
# Methode      : _ p r o p e r t y
# Klasse       : Graph::GED::Basis
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Private Funktion fr die einfache Definition von Gettern und
#                Settern.
# Parameter    : Attribut, Parameter
# Rckgabewert : Der aktuelle Wert (im Falle des Getters) oder der alte Wert
#                (im falle des Setters).
# -----------------------------------------------------------------------------
# 0.0.1 - 10.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self   = shift;    # Objekt
    my $attr   = shift;    # Attribute, das gesetzt oder geholt wird
    my $setter = @_ == 1;  # Ist die Methode Setter?
    my $value  = shift;    # Zu setzender Wert, falls Setter

    # Falls man hier "if (defined $value)" benutzen wrde, so knnte man keine
    # undefinierten Werte setzen, deshalb verwenden wir diese Form:
    if ($setter) {
        my $old_value = $self->{$attr};
        $self->{$attr} = $value;
        return $old_value;
    }
    else {
        return $self->{$attr};
    }
} # sub Basis::_property


sub clear {
# -----------------------------------------------------------------------------
# Methode      : c l e a r
# Klasse       : Graph::GED::Basis
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Lscht alle Ecken und Kanten.
# Parameter    : keine
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 09.02.2003 - CD - Erstellt
# 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
#                           Umstellung auf die Kantenklasse.
# -----------------------------------------------------------------------------

    my $self = shift;

    die "raph::GED::Basis::clear mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::GED::Basis::clear\n" if $Debug;

    #--------------------------------------------------------------------------
    # Flache Daten auf Null setzen:
    #--------------------------------------------------------------------------
    #$self->plandata(0);
    # keine da

    #--------------------------------------------------------------------------
    # Array-Daten lschen:
    #--------------------------------------------------------------------------
    #$self->_arraydata(undef);
    #$self->_arraydataecken([]);
    # keine da

    #--------------------------------------------------------------------------
    # Basisfunktionalitt aufrufen:
    #--------------------------------------------------------------------------
    #$self->SUPER::clear();
    # ist Basisklasse
} # sub raph::GED::Basis::clear


return 1;

__END__


=head1 NAME

Graphen::GED::Basis.pm - a base module for my other modules

=head1 METHODS

=head2 class methods

=over

=item *

C<new> - creates a new Ecke objekt

=back

=head2 objekt methods

=over

=item *

C<clone> - a method to clone this object

=back

=head1 COPYRIGHT

Copyright 2004 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
